@@ -1,6 +1,5 @@
 from copy import deepcopy
 
-import pytest
 from lxml import etree
 
 from translate.misc.multistring import multistring
@@ -48,18 +47,18 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
 
     def test_escape_message_with_newline(self):
         string = "message\nwith newline"
-        xml = '<string name="teststring">message\n\\nwith newline</string>\n'
+        xml = '<string name="teststring">message\\nwith newline</string>\n'
         self.__check_escape(string, xml)
 
     def test_escape_quotes_with_newline(self):
         string = "'message'\nwith newline"
-        xml = "<string name=\"teststring\">\\'message\\'\n\\nwith newline</string>\n"
+        xml = "<string name=\"teststring\">\\'message\\'\\nwith newline</string>\n"
         self.__check_escape(string, xml)
 
     def test_escape_message_with_newline_in_xml(self):
         string = "message\n\nwith newline in xml\n"
         xml = (
-            '<string name="teststring">message\n\\n\n\\nwith newline in xml\n\\n'
+            '<string name="teststring">message\\n\\nwith newline in xml\\n'
             "</string>\n"
         )
         self.__check_escape(string, xml)
@@ -112,7 +111,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
     def test_escape_html_code_quote_newline(self):
         string = "some \n<b>html code</b> 'here'"
         xml = (
-            "<string name=\"teststring\">some \n\\n<b>html code</b> \\'here\\'"
+            "<string name=\"teststring\">some \\n<b>html code</b> \\'here\\'"
             "</string>\n"
         )
         self.__check_escape(string, xml)
@@ -150,8 +149,8 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         )
         xml = (
             '<plurals name="teststring">\n'
-            '    <item quantity="one">one message\n\\nwith newline</item>'
-            '<item quantity="other">other message\n\\nwith newline</item>'
+            '    <item quantity="one">one message\\nwith newline</item>'
+            '<item quantity="other">other message\\nwith newline</item>'
             "</plurals>\n"
         )
         self.__check_escape(mString, xml, "en")
@@ -226,9 +225,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
     def test_escape_quoted_newlines(self):
         self.__check_escape(
             "\n\nstring with newlines",
-            r"""<string name="teststring">"
-\n
-\nstring with newlines"</string>
+            r"""<string name="teststring">\n\nstring with newlines</string>
 """,
         )
 
@@ -240,7 +237,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         self.__check_parse(string, xml)
 
     def test_parse_message_with_newline_in_xml(self):
-        string = "message\nwith\n newline\n in xml"
+        string = "message \nwith\n newline\n in xml"
         xml = (
             '<string name="teststring">message\n\\nwith\\n\nnewline\\n\nin xml'
             "</string>\n"
@@ -327,6 +324,16 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         xml = '<string name="teststring">test    </string>\n'
         self.__check_parse(string, xml)
 
+    def test_parse_trailing_escaped_newline(self):
+        string = "test\n"
+        xml = '<string name="teststring">test\\n</string>\n'
+        self.__check_parse(string, xml)
+
+    def test_parse_leading_escaped_newline(self):
+        string = "\ntest"
+        xml = '<string name="teststring">\\ntest</string>\n'
+        self.__check_parse(string, xml)
+
     def test_parse_leading_spaces(self):
         string = "test"
         xml = '<string name="teststring">    test</string>\n'
@@ -401,7 +408,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         self.__check_parse(string, xml)
 
     def test_parse_html_leading_space(self):
-        string = " <b>html code 'to escape'</b> some 'here'"
+        string = "<b>html code 'to escape'</b> some 'here'"
         xml = (
             "<string name=\"teststring\"> <b>html code \\'to escape\\'</b> some \\'here\\'"
             "</string>\n"
@@ -417,7 +424,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         self.__check_parse(string, xml)
 
     def test_parse_html_trailing_space(self):
-        string = "<b>html code 'to escape'</b> some 'here' "
+        string = "<b>html code 'to escape'</b> some 'here'"
         xml = (
             "<string name=\"teststring\"><b>html code \\'to escape\\'</b> some \\'here\\' "
             "</string>\n"
@@ -466,8 +473,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         self.__check_parse(string, xml)
 
     def test_parse_unicode(self):
-        with pytest.raises(ValueError):
-            self.__check_parse("", r'<string name="test">\utest</string>')
+        self.__check_parse("test", r'<string name="test">\utest</string>')
         self.__check_parse("\u0230", r'<string name="test">\u0230</string>')
 
     def test_single_unescaped(self):
@@ -889,6 +895,20 @@ files</strong> on the storage.</p>
         store.units[0].target = body
         assert bytes(store).decode() == content
 
+    def test_cdata_text(self):
+        body = """By continuing you accept our <![CDATA[<a href="%1$s">Terms of Use</a>]]> and <![CDATA[<a href="%2$s">Privacy Policy</a>]]>"""
+        content = f"""<?xml version="1.0" encoding="utf-8"?>
+<resources>
+    <string name="service_terms_agreement_notice">{body}</string>
+</resources>"""
+        store = self.StoreClass()
+        store.parse(content.encode())
+        assert len(store.units) == 1
+        assert store.units[0].target == body
+        assert bytes(store).decode() == content
+        store.units[0].target = body
+        assert bytes(store).decode() == content
+
     def test_prefix(self):
         body = "&lt; <b>body</b>"
         content = f"""<?xml version="1.0" encoding="utf-8"?>
@@ -952,6 +972,16 @@ files</strong> on the storage.</p>
         assert store.units[0].getid() == "test1"
         assert not store.units[0].istranslatable()
 
+    def test_escaping(self):
+        content = r"""<?xml version="1.0" encoding="utf-8" standalone="no"?>
+<resources>
+<string name="hello" weblate-flags="c-format, max-length:100">Hello, world!\n</string>
+</resources>"""
+        store = self.StoreClass()
+        store.parse(content.encode())
+        assert len(store.units) == 1
+        assert store.units[0].target == "Hello, world!\n"
+
 
 class TestMOKOResourceUnit(test_monolingual.TestMonolingualUnit):
     UnitClass = aresource.MOKOResourceUnit
