@@ -128,7 +128,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
 
     def propregen(self, propsource):
         """Helper that converts properties source to propfile object and back."""
-        return self.propparse(propsource).__bytes__()
+        return bytes(self.propparse(propsource))
 
     def test_quotes(self):
         """Checks that quotes are parsed and saved correctly."""
@@ -158,9 +158,9 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         propunit = propfile.units[0]
         assert propunit.name == "test_me"
         assert propunit.source.strings == ["I can code single!", "I can code!"]
-        assert propunit.value == ["I can code!", "I can code single!"]
+        assert propunit.value == ["I can code!"]
         propunit.value = ["I can code double!", "I can code single!"]
-        assert propunit.value == ["I can code double!", "I can code single!"]
+        assert propunit.value == ["I can code double!"]
         assert propunit.source.strings == ["I can code single!", "I can code double!"]
         # propunit.value = ["I can code single!", "I can code!" ]
         # assert propunit.value == ["I can code single!", "I can code!"]
@@ -179,7 +179,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         )  # Only "other" plural form
         print(propfile)
         print(str(propfile))
-        assert propfile.__bytes__() == b"test_me=I can code!\n"
+        assert (bytes(propfile)) == b"test_me=I can code!\n"
 
     def test_increase(self):
         """Checks that if the target language has more plural form the generated properties file is correct."""
@@ -193,7 +193,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         assert isinstance(propunit.target, multistring)
         assert propunit.target.strings == ["", "", "", "", "", ""]
         assert (
-            propfile.__bytes__()
+            (bytes(propfile))
             == b"test_me=I can code!\ntest_me[none]=\ntest_me[one]=I can code single!\n"
             b"test_me[two]=\ntest_me[few]=\ntest_me[many]=\n"
         )
@@ -209,7 +209,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         assert isinstance(propunit.target, multistring)
         assert propunit.target.strings == ["zero", "one", "two", "few", "many", "other"]
         assert (
-            propfile.__bytes__()
+            (bytes(propfile))
             == b"test_me=other\ntest_me[none]=zero\ntest_me[one]=one\n"
             b"test_me[two]=two\ntest_me[few]=few\ntest_me[many]=many\n"
         )
@@ -218,7 +218,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         assert isinstance(propunit.target, multistring)
         assert propunit.target.strings == ["zero", "one", "two", "few", "many", "other"]
         assert (
-            propfile.__bytes__()
+            (bytes(propfile))
             == b"test_me=other\ntest_me[none]=zero\ntest_me[one]=one\n"
             b"test_me[two]=two\ntest_me[few]=few\ntest_me[many]=many\n"
         )
@@ -227,7 +227,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         assert isinstance(propunit.target, multistring)
         assert propunit.target.strings == ["zero", "one", "two", "few", "many", "other"]
         assert (
-            propfile.__bytes__()
+            (bytes(propfile))
             == b"test_me=other\ntest_me[none]=zero\ntest_me[one]=one\n"
             b"test_me[two]=two\ntest_me[few]=few\ntest_me[many]=many\n"
         )
@@ -236,7 +236,7 @@ class TestGwtProp(test_monolingual.TestMonolingualStore):
         assert isinstance(propunit.target, multistring)
         assert propunit.target.strings == ["zero", "one", "two", "few", "many", "other"]
         assert (
-            propfile.__bytes__()
+            (bytes(propfile))
             == b"test_me=other\ntest_me[none]=zero\ntest_me[one]=one\n"
             b"test_me[two]=two\ntest_me[few]=few\ntest_me[many]=many\n"
         )
@@ -294,6 +294,62 @@ PP[]=Other
         propunit.source = "xZkouška"
         assert bytes(propfile).decode() == "test=xZkou\\u0161ka\n"
 
+    def test_other_plurals(self):
+        propsource = r"""userItems.limit=Only X items can be added.
+userItems.limit[one]=Only one item can be added.
+userItems.limit[few]=Only {0} items can be added.
+userItems.limit[many]=Only {0} items can be added.
+"""
+        propfile = self.propparse(propsource, sourcelanguage="pl", targetlanguage="pl")
+        propsource_en = r"""
+userItems.test={0} items can be added.
+userItems.test[one]=One item can be added.
+"""
+        propfile_en = self.propparse(
+            propsource_en, sourcelanguage="en", targetlanguage="en"
+        )
+
+        assert len(propfile_en.units) == 1
+        assert len(propfile.units) == 1
+        propunit = propfile.units[0]
+        assert propunit.name == "userItems.limit"
+        assert isinstance(propunit.target, multistring)
+        assert propunit.source.strings == [
+            "Only one item can be added.",
+            "Only {0} items can be added.",
+            "Only {0} items can be added.",
+        ]
+        propunit.target = multistring(
+            [
+                "Only one item can be added.",
+                "Only {0} items can be added.",
+                "Only many {0} items can be added.",
+            ]
+        )
+
+        unit = propfile_en.units[0]
+        propfile.addunit(unit)
+        unit.target = multistring(
+            [
+                "Only one item can be added.",
+                "Only {0} items can be added.",
+                "Only many {0} items can be added.",
+            ]
+        )
+
+        assert (
+            bytes(propfile).decode()
+            == """userItems.limit=Only many {0} items can be added.
+userItems.limit[one]=Only one item can be added.
+userItems.limit[few]=Only {0} items can be added.
+userItems.limit[many]=Only many {0} items can be added.
+userItems.test=Only many {0} items can be added.
+userItems.test[one]=Only one item can be added.
+userItems.test[few]=Only {0} items can be added.
+userItems.test[many]=Only many {0} items can be added.
+"""
+        )
+
 
 class TestProp(test_monolingual.TestMonolingualStore):
     StoreClass = properties.propfile
@@ -806,6 +862,15 @@ key=value
         assert propunit.value == "value"
         assert propunit.getnotes() == "long\nnote"
 
+    def test_trailing_newlines(self):
+        """Ensure we can handle Unicode."""
+        propsource = """"I am a “key”" = "I am a “value”";\n"""
+        propfile = self.propparse(
+            propsource.encode() + b"\n" * 10, personality="strings-utf8"
+        )
+        assert len(propfile.units) == 1
+        assert bytes(propfile).decode() == propsource
+
 
 class TestXWiki(test_monolingual.TestMonolingualStore):
     StoreClass = properties.xwikifile
