@@ -25,7 +25,7 @@ from translate.lang import data
 
 
 def spacestart(str1):
-    """returns all the whitespace from the start of the string"""
+    """Returns all the whitespace from the start of the string."""
     newstring = ""
     for c in str1:
         if c.isspace():
@@ -36,7 +36,7 @@ def spacestart(str1):
 
 
 def spaceend(str1):
-    """returns all the whitespace from the end of the string"""
+    """Returns all the whitespace from the end of the string."""
     newstring = ""
     for n in range(len(str1)):
         c = str1[-1 - n]
@@ -48,7 +48,7 @@ def spaceend(str1):
 
 
 def puncstart(str1, punctuation):
-    """returns all the punctuation from the start of the string"""
+    """Returns all the punctuation from the start of the string."""
     newstring = ""
     for c in str1:
         if c in punctuation or c.isspace():
@@ -59,7 +59,7 @@ def puncstart(str1, punctuation):
 
 
 def puncend(str1, punctuation):
-    """returns all the punctuation from the end of the string"""
+    """Returns all the punctuation from the end of the string."""
     # An implementation with regular expressions was slightly slower.
 
     newstring = ""
@@ -73,7 +73,7 @@ def puncend(str1, punctuation):
 
 
 def ispurepunctuation(str1):
-    """checks whether the string is entirely punctuation"""
+    """Checks whether the string is entirely punctuation."""
     for c in str1:
         if c.isalnum():
             return False
@@ -81,7 +81,8 @@ def ispurepunctuation(str1):
 
 
 def isvalidaccelerator(accelerator, acceptlist=None):
-    """returns whether the given accelerator character is valid
+    """
+    returns whether the given accelerator character is valid.
 
     :type accelerator: character
     :param accelerator: A character to be checked for accelerator validity
@@ -97,29 +98,27 @@ def isvalidaccelerator(accelerator, acceptlist=None):
         return False
     if acceptlist is not None:
         acceptlist = data.normalize(acceptlist)
-        if accelerator in acceptlist:
-            return True
+        return accelerator in acceptlist
+    # Old code path - ensures that we don't get a large number of
+    # regressions
+    accelerator = accelerator.replace("_", "")
+    if accelerator in "-?":
+        return True
+    if not accelerator.isalnum():
         return False
-    else:
-        # Old code path - ensures that we don't get a large number of
-        # regressions
-        accelerator = accelerator.replace("_", "")
-        if accelerator in "-?":
-            return True
-        if not accelerator.isalnum():
-            return False
 
-        # We don't want to have accelerators on characters with diacritics,
-        # so let's see if the character can decompose.
-        decomposition = unicodedata.decomposition(accelerator)
-        # Next we strip out any extra information like <this>
-        decomposition = re.sub("<[^>]+>", "", decomposition).strip()
-        return decomposition.count(" ") == 0
+    # We don't want to have accelerators on characters with diacritics,
+    # so let's see if the character can decompose.
+    decomposition = unicodedata.decomposition(accelerator)
+    # Next we strip out any extra information like <this>
+    decomposition = re.sub(r"<[^>]+>", "", decomposition).strip()
+    return decomposition.count(" ") == 0
 
 
 def findaccelerators(str1, accelmarker, acceptlist=None):
-    """returns all the accelerators and locations in str1 marked with a given
-    marker
+    """
+    returns all the accelerators and locations in str1 marked with a given
+    marker.
     """
     accelerators = []
     badaccelerators = []
@@ -143,8 +142,9 @@ def findaccelerators(str1, accelmarker, acceptlist=None):
 
 
 def findmarkedvariables(str1, startmarker, endmarker, ignorelist=[]):
-    """returns all the variables and locations in str1 marked with a given
-    marker
+    """
+    returns all the variables and locations in str1 marked with a given
+    marker.
     """
     variables = []
     currentpos = 0
@@ -167,7 +167,7 @@ def findmarkedvariables(str1, startmarker, endmarker, ignorelist=[]):
                 if currentpos < endmatch:
                     variable = str1[currentpos:endmatch]
                 currentpos = endmatch
-            elif type(endmarker) is int:
+            elif isinstance(endmarker, int):
                 # setting endmarker to an int means it is a fixed-length
                 # variable string (usually endmarker==1)
                 endmatch = currentpos + endmarker
@@ -197,12 +197,13 @@ def findmarkedvariables(str1, startmarker, endmarker, ignorelist=[]):
 
 
 def getaccelerators(accelmarker, acceptlist=None):
-    """returns a function that gets a list of accelerators marked using
-    accelmarker
+    """
+    returns a function that gets a list of accelerators marked using
+    accelmarker.
     """
 
     def getmarkedaccelerators(str1):
-        """returns all the accelerators in str1 marked with a given marker"""
+        """Returns all the accelerators in str1 marked with a given marker."""
         acclocs, badlocs = findaccelerators(str1, accelmarker, acceptlist)
         accelerators = [accelerator for accelstart, accelerator in acclocs]
         badaccelerators = [accelerator for accelstart, accelerator in badlocs]
@@ -212,12 +213,13 @@ def getaccelerators(accelmarker, acceptlist=None):
 
 
 def getvariables(startmarker, endmarker):
-    """returns a function that gets a list of variables marked using
-    startmarker and endmarker
+    """
+    returns a function that gets a list of variables marked using
+    startmarker and endmarker.
     """
 
     def getmarkedvariables(str1):
-        """returns all the variables in str1 marked with a given marker"""
+        """Returns all the variables in str1 marked with a given marker."""
         varlocs = findmarkedvariables(str1, startmarker, endmarker)
         return [variable for accelstart, variable in varlocs]
 
@@ -225,7 +227,7 @@ def getvariables(startmarker, endmarker):
 
 
 def getnumbers(str1):
-    """returns any numbers that are in the string"""
+    """Returns any numbers that are in the string."""
     # TODO: handle locale-based periods e.g. 2,5 for Afrikaans
     assert isinstance(str1, str)
     numbers = []
@@ -237,7 +239,7 @@ def getnumbers(str1):
         if chr1.isdigit():
             innumber = True
         elif innumber:
-            if chr1 not in (".", degreesign):
+            if chr1 not in {".", degreesign}:
                 innumber = False
                 if lastnumber:
                     numbers.append(lastnumber)
@@ -258,6 +260,8 @@ def getnumbers(str1):
     return numbers
 
 
+_email_re = re.compile(r"[\w\.\-]+@[\w\.\-]+")
+
 _function_re = re.compile(
     r"""((?:
     [\w\.]+              # function or module name - any alpha-numeric character, _, or .
@@ -274,38 +278,39 @@ _function_re = re.compile(
 
 
 def getfunctions(str1):
-    """returns the functions() that are in a string, while ignoring the
-    trailing punctuation in the given parameter
+    """
+    returns the functions() that are in a string, while ignoring the
+    trailing punctuation in the given parameter.
     """
     if "()" in str1:
         return _function_re.findall(str1)
-    else:
-        return []
+    return []
 
 
 def getemails(str1):
-    """returns the email addresses that are in a string"""
-    return re.findall(r"[\w\.\-]+@[\w\.\-]+", str1)
+    """Returns the email addresses that are in a string."""
+    return _email_re.findall(str1)
 
 
 def geturls(str1):
-    """returns the URIs in a string"""
+    """Returns the URIs in a string."""
     # TODO turn this into a verbose and compiled regex
     URLPAT = (
         r"https?:[\w/\.:;+\-~\%#\$?=&,()]+|"
-        + r"www\.[\w/\.:;+\-~\%#\$?=&,()]+|"
-        + r"ftp:[\w/\.:;+\-~\%#?=&,]+"
+        r"www\.[\w/\.:;+\-~\%#\$?=&,()]+|"
+        r"ftp:[\w/\.:;+\-~\%#?=&,]+"
     )
     return re.findall(URLPAT, str1)
 
 
 def countaccelerators(accelmarker, acceptlist=None):
-    """returns a function that counts the number of accelerators marked with
-    the given marker
+    """
+    returns a function that counts the number of accelerators marked with
+    the given marker.
     """
 
     def countmarkedaccelerators(str1):
-        """returns all the variables in str1 marked with a given marker"""
+        """Returns all the variables in str1 marked with a given marker."""
         acclocs, badlocs = findaccelerators(str1, accelmarker, acceptlist)
         return len(acclocs), len(badlocs)
 
