@@ -40,7 +40,7 @@ class ProgressBar:
 
     def __init__(self, progress_type, allfiles):
         """Set up a progress bar appropriate to the progress_type and files."""
-        if progress_type in ("bar", "verbose"):
+        if progress_type in {"bar", "verbose"}:
             file_count = len(allfiles)
             self._progressbar = self.progress_types[progress_type](0, file_count)
             logger = logging.getLogger(os.path.basename(sys.argv[0])).getChild(
@@ -163,37 +163,43 @@ class RecursiveOptionParser(optparse.OptionParser):
         def formatToolkit(x):
             return x.replace("%prog", "Translate Toolkit")
 
-        result.append('.\\" Autogenerated manpage\n')
-        result.append(
-            f'.TH {prog} 1 "{formatToolkit(self.version)}" "" "{formatToolkit(self.version)}"\n'
-        )
-        result.append(".SH NAME\n")
-        result.append(
-            "{} \\- {}\n".format(
-                self.get_prog_name(), self.description.split("\n\n")[0]
+        result.extend(
+            (
+                '.\\" Autogenerated manpage\n',
+                f'.TH {prog} 1 "{formatToolkit(self.version)}" "" "{formatToolkit(self.version)}"\n',
+                ".SH NAME\n",
+                "{} \\- {}\n".format(
+                    self.get_prog_name(), self.description.split("\n\n")[0]
+                ),
+                ".SH SYNOPSIS\n",
+                ".PP\n",
             )
         )
-        result.append(".SH SYNOPSIS\n")
-        result.append(".PP\n")
         usage = "\\fB%prog "
         usage += " ".join(self.getusageman(option) for option in self.option_list)
         usage += "\\fP"
         result.append(f"{formatprog(usage)}\n")
         description_lines = self.description.split("\n\n")[1:]
         if description_lines:
-            result.append(".SH DESCRIPTION\n")
-            result.append(
-                "\n\n".join(
-                    re.sub(r"\.\. note::", "Note:", l) for l in description_lines
+            result.extend(
+                (
+                    ".SH DESCRIPTION\n",
+                    "\n\n".join(
+                        re.sub(r"\.\. note::", "Note:", l) for l in description_lines
+                    ),
                 )
             )
         result.append(".SH OPTIONS\n")
         ManHelpFormatter().store_option_strings(self)
         result.append(".PP\n")
         for option in self.option_list:
-            result.append(".TP\n")
-            result.append("{}\n".format(str(option).replace("-", r"\-")))
-            result.append("{}\n".format(option.help.replace("-", r"\-")))
+            result.extend(
+                (
+                    ".TP\n",
+                    "{}\n".format(str(option).replace("-", "\\-")),
+                    "{}\n".format(option.help.replace("-", "\\-")),
+                )
+            )
         return "".join(result)
 
     def print_manpage(self, file=None):
@@ -313,7 +319,7 @@ class RecursiveOptionParser(optparse.OptionParser):
                 outputformats.append(outputformat)
             if templateformat not in templateformats:
                 templateformats.append(templateformat)
-            self.outputoptions[(inputformat, templateformat)] = (
+            self.outputoptions[inputformat, templateformat] = (
                 outputformat,
                 processor,
             )
@@ -667,7 +673,7 @@ class RecursiveOptionParser(optparse.OptionParser):
     ):
         """Process an individual file."""
         inputfile = self.openinputfile(options, fullinputpath)
-        if fulloutputpath and fulloutputpath in (fullinputpath, fulltemplatepath):
+        if fulloutputpath and fulloutputpath in {fullinputpath, fulltemplatepath}:
             outputfile = self.opentempoutputfile(options, fulloutputpath)
             tempoutput = True
         else:
@@ -812,7 +818,7 @@ class RecursiveOptionParser(optparse.OptionParser):
                             return templatepath
             if "*" in self.inputformats:
                 for inputext1, templateext1 in self.outputoptions:
-                    if inputext1 in (inputext, "*"):
+                    if inputext1 in {inputext, "*"}:
                         if templateext1 == "*":
                             templatepath = inputname
                             if self.templateexists(options, templatepath):
