@@ -32,19 +32,19 @@ from translate.lang import data
 from translate.misc.multistring import multistring
 from translate.storage import base, lisa
 
-EOF = None
 WHITESPACE = {" ", "\n", "\t"}  # Whitespace that we collapse.
 ESCAPE_NEWLINE = {"n", "N"}
 ESCAPE_TAB = {"t", "T"}
 ESCAPE_PLAIN = {" ", '"', "'", "@", "?"}
-MULTIWHITESPACE = re.compile("[ \n\t]{2}(?!\\\\n)")
+MULTIWHITESPACE = re.compile(r"[ \n\t]{2}(?!\\n)")
+QUOTED_STRING = re.compile(r'("[^"]*")')
+UNICODE_ESCAPE = re.compile(r"\\u([a-fA-F0-9]{4})")
+CHAR_ESCAPE = re.compile(r"\\(.)")
 
 ESCAPE_TRANSLATE = str.maketrans(
     {
         "\\": "\\\\",
-        # This will add non intrusive real newlines to
-        # ones in translation improving readability of result
-        "\n": "\n\\n",
+        "\n": "\\n",
         "\t": "\\t",
         "'": "\\'",
         '"': '\\"',
@@ -71,6 +71,8 @@ class DecodingXMLParser:
         self.decoded_emit = None
         self.character_data = False
         self.raw_string = True
+        self.in_string = False
+        self.do_cleanup = False
         self.depth = 0
         self.parser = parser = ParserCreate()
         if self.FOREGIN_DTD:
@@ -84,20 +86,80 @@ class DecodingXMLParser:
         parser.DefaultHandler = self.DefaultHandler
 
     @staticmethod
-    def process_string(text: str) -> str:
-        return AndroidResourceUnit.unescape(text, strip=False)
+    def decode_unicode_escapes(match: re.Match) -> str:
+        escaped = match.group(1)
+        # Convert from hex to character
+        return chr(int(escaped, 16))
+
+    @staticmethod
+    def decode_escapes(match: re.Match) -> str:
+        escaped = match.group(1)
+        if escaped in ESCAPE_NEWLINE:
+            return "\n"  # an actual newline
+        if escaped in ESCAPE_TAB:
+            return "\t"  # an actual tab
+        if escaped in ESCAPE_PLAIN:
+            # Plain string to escape
+            return escaped
+        # All others, remove, like Android does as well.
+        return ""
+
+    @classmethod
+    def process_string(cls, content: str) -> tuple[str, bool, bool]:
+        # Skip processing for a blank string
+        if not content:
+            return "", False, False
+
+        # Extract quoted parts, remove blank ones
+        parts = [part for part in QUOTED_STRING.split(content) if part]
+
+        # Detect possible cleanups
+        cleanup_start = not parts[0].startswith('"')
+        cleanup_end = not parts[-1].startswith('"')
+
+        # Collapse whitespace
+        stripped_parts = [
+            re.sub(r"\s+", " ", part) if not part.startswith('"') else part[1:-1]
+            for part in parts
+        ]
+        # Merge this back to a single string
+        text = "".join(stripped_parts)
+
+        # Handle unicode escapes
+        unescaped_text = UNICODE_ESCAPE.sub(cls.decode_unicode_escapes, text)
+        # Handle other escapes
+        unescaped_text = CHAR_ESCAPE.sub(cls.decode_escapes, unescaped_text)
+
+        # Detect escaping at the start and end of a string (we actually care only for
+        # whitespace escapes, but detecting more does not matter)
+        if text and text[0] != unescaped_text[0]:
+            cleanup_start = False
+        if text and text[-1] != unescaped_text[-1]:
+            cleanup_end = False
+
+        return unescaped_text, cleanup_start, cleanup_end
+
+    def cleanup_text(self, text: str) -> str:
+        """Remove leading whitespace from text."""
+        if not self.output:
+            return text.lstrip()
+        return text
 
     def emit(self):
         if self.emit_start is not None:
             text = self.text[self.emit_start : self.parser.CurrentByteIndex].decode(
                 "utf-8"
             )
+            self.do_cleanup = not self.raw_string
             if not self.raw_string:
-                text = self.process_string(text)
+                text, cleanup_start, self.do_cleanup = self.process_string(text)
+                if cleanup_start:
+                    text = self.cleanup_text(text)
             self.output.append(text)
             self.emit_start = None
         self.character_data = False
         self.raw_string = True
+        self.in_string = False
 
     def StartElementHandler(self, _name, _attributes):
         self.emit()
@@ -112,10 +174,11 @@ class DecodingXMLParser:
         self.depth -= 1
 
     def CharacterDataHandler(self, _data):
-        if not self.character_data:
+        if not self.character_data and not self.in_string:
             self.emit()
             self.emit_start = self.parser.CurrentByteIndex
             self.raw_string = False
+            self.in_string = True
 
     def StartCdataSectionHandler(self):
         self.emit()
@@ -134,6 +197,9 @@ class DecodingXMLParser:
         self.parser.Parse(self.text, True)
         if self.depth >= self.EMIT_DEPTH:
             self.emit()
+        # Remove trailing whitespace from text
+        if self.do_cleanup and self.output:
+            self.output[-1] = self.output[-1].rstrip()
         return "".join(self.output)
 
 
@@ -142,8 +208,12 @@ class EncodingXMLParser(DecodingXMLParser):
     FOREGIN_DTD = False
 
     @staticmethod
-    def process_string(text: str) -> str:
-        return AndroidResourceUnit.escape(text, quote_wrapping_whitespaces=False)
+    def process_string(text: str) -> tuple[str, bool, bool]:
+        return (
+            AndroidResourceUnit.escape(text, quote_wrapping_whitespaces=False),
+            False,
+            False,
+        )
 
     def parse(self) -> str:
         self.emit_start = 0
@@ -193,131 +263,6 @@ class AndroidResourceUnit(base.TranslationUnit):
     def getcontext(self):
         return self.xmlelement.get("name")
 
-    @staticmethod
-    def unescape(text, strip=True):
-        """
-        Remove escaping from Android resource.
-
-        Code is based on android2po
-        <https://github.com/miracle2k/android2po>
-        """
-        # Return text for empty elements
-        if text is None:
-            return ""
-
-        # Mirror globals into locals for faster lookups
-        eof = EOF
-        whitespace = WHITESPACE
-        escape_newline = ESCAPE_NEWLINE
-        escape_tab = ESCAPE_TAB
-        escape_plain = ESCAPE_PLAIN
-
-        # We need to collapse multiple whitespace while paying
-        # attention to Android's quoting and escaping.
-        space_count = 0
-        active_quote = False
-        active_escape = False
-        i = 0
-        text = [*text, eof]
-        while i < len(text):
-            c = text[i]
-
-            if not active_escape:
-                # Handle whitespace collapsing
-                if c is not eof and c in whitespace:
-                    space_count += 1
-                elif space_count >= 1:
-                    # Remove sequential whitespace; Pay attention: We
-                    # don't do this if we are currently inside a quote,
-                    # except for one special case: If we have unbalanced
-                    # quotes, e.g. we reach eof while a quote is still
-                    # open, we *do* collapse that trailing part; this is
-                    # how Android does it, for some reason.
-                    if not active_quote or c is eof:
-                        # Replace by a single space, will get rid of
-                        # non-significant newlines/tabs etc.
-                        text[i - space_count : i] = " "
-                        i -= space_count - 1
-                        if strip and (i == 1 or c is eof):
-                            del text[i - 1]
-                    space_count = 0
-                else:
-                    space_count = 0
-
-            # Handle quotes
-            if c == '"' and not active_escape:
-                active_quote = not active_quote
-                del text[i]
-                i -= 1
-
-            # Handle escapes
-            if c == "\\":
-                if not active_escape:
-                    active_escape = True
-                else:
-                    # A double-backslash represents a single;
-                    # simply deleting the current char will do.
-                    del text[i]
-                    i -= 1
-                    active_escape = False
-            elif active_escape:
-                # Handle the limited amount of escape codes
-                # that we support.
-                # TODO: What about \r, or \r\n?
-                if c is eof:
-                    # Basically like any other char, but put
-                    # this first so we can use the ``in`` operator
-                    # in the clauses below without issue.
-                    pass
-                elif c in escape_newline:
-                    # Remove whitespace just before newline. Most likely this is result of
-                    # having real newline in the XML in front of \n.
-                    offset = 2 if i >= 2 and text[i - 2] == " " else 1
-                    text[i - offset : i + 1] = "\n"  # an actual newline
-                    i -= offset
-                elif c in escape_tab:
-                    text[i - 1 : i + 1] = "\t"  # an actual tab
-                    i -= 1
-                elif c in escape_plain:
-                    text[i - 1 : i] = ""  # remove the backslash
-                    i -= 1
-                elif c == "u":
-                    # Unicode sequence. Android is nice enough to deal
-                    # with those in a way which let's us just capture
-                    # the next 4 characters and raise an error if they
-                    # are not valid (rather than having to use a new
-                    # state to parse the unicode sequence).
-                    # Exception: In case we are at the end of the
-                    # string, we support incomplete sequences by
-                    # prefixing the missing digits with zeros.
-                    # Note: max(len()) is needed in the slice due to
-                    # trailing ``None`` element.
-                    max_slice = min(i + 5, len(text) - 1)
-                    codepoint_str = "".join(text[i + 1 : max_slice])
-                    if len(codepoint_str) < 4:
-                        codepoint_str = "0" * (4 - len(codepoint_str)) + codepoint_str
-                    try:
-                        # We can't trust int() to raise a ValueError,
-                        # it will ignore leading/trailing whitespace.
-                        if not codepoint_str.isalnum():
-                            raise ValueError(codepoint_str)
-                        codepoint = chr(int(codepoint_str, 16))
-                    except ValueError:
-                        raise ValueError("bad unicode escape sequence")
-
-                    text[i - 1 : max_slice] = codepoint
-                    i -= 1
-                else:
-                    # All others, remove, like Android does as well.
-                    text[i - 1 : i + 1] = ""
-                    i -= 1
-                active_escape = False
-
-            i += 1
-
-        # Join the string together again, but w/o EOF marker
-        return "".join(text[:-1])
-
     @staticmethod
     def xml_escape_space(matchobj):
         return matchobj.group(0).replace("  ", r" \u0020")
@@ -367,13 +312,23 @@ class AndroidResourceUnit(base.TranslationUnit):
         return super().source
 
     def get_xml_text_value(self, xmltarget):
-        if len(xmltarget) == 0:
-            # There are no html markups, so unescaping it as plain text.
-            return self.unescape(xmltarget.text)
-        parser = DecodingXMLParser(
-            etree.tostring(xmltarget, encoding="unicode", with_tail=False)
-        )
+        raw_xml = etree.tostring(xmltarget, encoding="unicode", with_tail=False)
+        # Unescape as plain text in case there is no XML markup. Ufortunately
+        # CDATA elements are not listed as childs in lxml, so test for it in raw XML.
+        if len(xmltarget) == 0 and "<![CDATA[" not in raw_xml:
+            if xmltarget.text is None:
+                return ""
+            text, cleanup_start, cleanup_end = DecodingXMLParser.process_string(
+                xmltarget.text
+            )
+            if cleanup_start:
+                text = text.lstrip()
+            if cleanup_end:
+                text = text.rstrip()
+            return text
 
+        # Use decoding parser to keep XML entitities, CDATA while processing Android escaping
+        parser = DecodingXMLParser(raw_xml)
         return parser.parse()
 
     def set_xml_text_plain(self, target, xmltarget):
@@ -401,7 +356,8 @@ class AndroidResourceUnit(base.TranslationUnit):
             else:
                 template = "<resources></resources>"
             newstring = template.replace(
-                "</resources>", f"<string>{target}</string></resources>"
+                "</resources>",
+                f"<{xmltarget.tag}>{target}</{xmltarget.tag}></resources>",
             )
             try:
                 etree.fromstring(newstring, parser)
@@ -412,15 +368,17 @@ class AndroidResourceUnit(base.TranslationUnit):
                 encoding_parser = EncodingXMLParser(newstring)
                 newstring = encoding_parser.parse()
                 newtree = etree.fromstring(newstring, parser)[0]
+                # Copy existing attributes
+                for attribute, value in xmltarget.items():
+                    newtree.attrib[attribute] = value
                 # Update text
-                xmltarget.text = newtree.text
-
-                # Remove old elements
-                for x in xmltarget.iterchildren():
-                    xmltarget.remove(x)
-                # Add new elements
-                for x in newtree.iterchildren():
-                    xmltarget.append(x)
+                parent = xmltarget.getparent()
+                # Parent can be none when operating on Unit only without a storage
+                if parent is not None:
+                    parent.replace(xmltarget, newtree)
+                # Update unit xmlelement if needed (plurals are inner elements here)
+                if xmltarget == self.xmlelement:
+                    self.xmlelement = newtree
         else:
             self.set_xml_text_plain(target, xmltarget)
 
@@ -434,8 +392,7 @@ class AndroidResourceUnit(base.TranslationUnit):
         if not locale:
             return data.plural_tags["en"]
         # Handle b+ style language codes
-        if locale.startswith("b+"):
-            locale = locale[2:]
+        locale = locale.removeprefix("b+")
         locale = locale.replace("_", "-").replace("+", "-").split("-")[0]
         return data.plural_tags.get(locale, data.plural_tags["en"])
 
@@ -480,8 +437,8 @@ class AndroidResourceUnit(base.TranslationUnit):
             for plural_tag, plural_string in zip(plural_tags, plural_strings):
                 item = etree.Element("item")
                 item.set("quantity", plural_tag)
-                self.set_xml_text_value(plural_string, item)
                 self.xmlelement.append(item)
+                self.set_xml_text_value(plural_string, item)
         else:
             # Fix the root tag if mismatching
             if self.xmlelement.tag != self.SINGULAR_TAG:
