@@ -276,7 +276,7 @@ class Dialect:
         """Encode the string."""
         # FIXME: dialects are a bad idea, not possible for subclasses
         # to override key methods
-        if encoding not in ("utf-8", "utf-16"):
+        if encoding not in {"utf-8", "utf-16"}:
             return quote.javapropertiesencode(string or "")
         return quote.java_utf8_properties_encode(string or "")
 
@@ -306,7 +306,7 @@ class Dialect:
         delimiters = delimiter_dict
         # Figure out starting position
         start_pos = len(line) - len(line.lstrip())  # Skip initial whitespace
-        if cls.key_wrap_char != "" and line[start_pos] == cls.key_wrap_char:
+        if cls.key_wrap_char and line[start_pos] == cls.key_wrap_char:
             # Skip the key if it is delimited by some char
             start_pos += 1
             while line[start_pos] != cls.key_wrap_char or line[start_pos - 1] == "\\":
@@ -369,6 +369,10 @@ class Dialect:
     def get_cldr_names_order():
         return ["other"]
 
+    @staticmethod
+    def get_expand_output_target_mapping(names: list[str]) -> list[str]:
+        return names
+
 
 @register_dialect
 class DialectJava(Dialect):
@@ -480,7 +484,7 @@ class DialectGwt(DialectJavaUtf8):
 
     @classmethod
     def get_cldr_names_order(cls):
-        return [y for x, y in cls.gwt_plural_categories]
+        return [y for _x, y in cls.gwt_plural_categories]
 
     @classmethod
     def get_key(cls, key, variant):
@@ -493,7 +497,7 @@ class DialectGwt(DialectJavaUtf8):
 
     @classmethod
     def encode(cls, string, encoding=None):
-        if encoding not in ("utf-8", "utf-16"):
+        if encoding not in {"utf-8", "utf-16"}:
             result = quote.javapropertiesencode(string or "")
         else:
             result = quote.java_utf8_properties_encode(string or "")
@@ -504,6 +508,12 @@ class DialectGwt(DialectJavaUtf8):
         result = super().decode(string)
         return result.replace("''", "'")
 
+    @staticmethod
+    def get_expand_output_target_mapping(names: list[str]) -> list[str]:
+        if "other" not in names:
+            return ["other", *names]
+        return names
+
 
 @register_dialect
 class DialectSkype(Dialect):
@@ -562,9 +572,7 @@ class DialectStrings(Dialect):
     @staticmethod
     def is_line_continuation(line):
         l = line.rstrip()
-        if l and l[-1] == ";":
-            return False
-        return True
+        return not l or l[-1] != ";"
 
     @staticmethod
     def strip_line_continuation(value):
@@ -642,7 +650,9 @@ class proppluralunit(base.TranslationUnit):
 
     def _get_ordered_units(self):
         # Used for str (GWT order)
-        mapping = self._get_target_mapping()
+        mapping = self.personality.get_expand_output_target_mapping(
+            self._get_target_mapping()
+        )
         names = [
             name for name in self.personality.get_cldr_names_order() if name in mapping
         ]
@@ -656,7 +666,7 @@ class proppluralunit(base.TranslationUnit):
     def settarget(self, text):
         mapping = None
         if isinstance(text, multistring):
-            strings = text.strings
+            strings = [str(x) for x in text.strings]
         elif isinstance(text, list):
             strings = text
         elif isinstance(text, dict):
@@ -672,6 +682,8 @@ class proppluralunit(base.TranslationUnit):
             raise ValueError(
                 f'Not same plural counts between "{strings}" and "{units}"'
             )
+        if "other" not in mapping and "other" in self.units:
+            self.units["other"].target = strings[-1]
 
         for a, b in zip(strings, units):
             b.target = a
@@ -893,7 +905,7 @@ class propunit(base.TranslationUnit):
 
     @property
     def target(self):
-        return re.sub("\\\\ ", " ", self.personality.decode(self.translation))
+        return re.sub(r"\\ ", " ", self.personality.decode(self.translation))
 
     @target.setter
     def target(self, target):
@@ -942,7 +954,7 @@ class propunit(base.TranslationUnit):
         return [self.name]
 
     def addnote(self, text, origin=None, position="append"):
-        if origin in ["programmer", "developer", "source code", None]:
+        if origin in {"programmer", "developer", "source code", None}:
             if get_comment_one_line(text) is None and get_comment_start(text) is None:
                 text = f"/* {text} */" if "\n" in text else f"// {text}"
             self.comments.append(text)
@@ -950,7 +962,7 @@ class propunit(base.TranslationUnit):
             super().addnote(text, origin=origin, position=position)
 
     def getnotes(self, origin=None):
-        if origin in ["programmer", "developer", "source code", None]:
+        if origin in {"programmer", "developer", "source code", None}:
             output = []
             inmultilinecomment = False
             for line in self.comments:
@@ -1133,11 +1145,7 @@ class propfile(base.TranslationStore):
                         self.addunit(newunit)
                         newunit = self.UnitClass("", self.personality.name)
         # see if there is a leftover one...
-        if (
-            inmultilinevalue
-            or len(newunit.comments) > 0
-            and not (len(newunit.comments) == 1 and not (newunit.comments[0]))
-        ):
+        if inmultilinevalue or any(newunit.comments):
             self.addunit(newunit)
 
         if self.personality.has_plurals:
@@ -1355,6 +1363,8 @@ class XWikiPageProperties(xwikifile):
         # if we are editing the source file we should not modify it.
         if not self.is_source_file():
             self.set_xwiki_xml_attributes(newroot)
+            # We only want a single line break before the closing node.
+            newroot.find("content").tail = "\n"
         self.write_xwiki_xml(newroot, out)
 
 
@@ -1374,9 +1384,9 @@ class XWikiFullPage(XWikiPageProperties):
             content = "".join(self.root.find("content").itertext()).replace("\n", "\\n")
             title = "".join(self.root.find("title").itertext())
             forparsing = ""
-            if content != "":
+            if content:
                 forparsing += f"content={content}\n"
-            if title != "":
+            if title:
                 forparsing += f"title={title}\n"
             self.extract_language()
             super(XWikiPageProperties, self).parse(forparsing.encode(self.encoding))
