@@ -29,9 +29,10 @@ import copy
 import logging
 import re
 import textwrap
-import unicodedata
 from itertools import chain
 
+from cwcwidth import wcswidth, wcwidth
+
 from translate.misc import quote
 from translate.misc.multistring import multistring
 from translate.storage import pocommon, poparser
@@ -44,8 +45,16 @@ lsep = "\n#: "
 
 # general functions for quoting / unquoting po strings
 
-po_unescape_map = {"\\r": "\r", "\\t": "\t", '\\"': '"', "\\n": "\n", "\\\\": "\\"}
+po_unescape_map = {
+    r"\r": "\r",
+    r"\t": "\t",
+    r"\"": '"',
+    r"\n": "\n",
+    r"\\": "\\",
+}
+po_unescape_re = re.compile(r"\\.")
 po_escape_map = {value: key for (key, value) in po_unescape_map.items()}
+po_escape_re = re.compile("|".join(re.escape(key) for key in po_escape_map))
 
 
 def splitlines(text):
@@ -78,52 +87,28 @@ def splitlines(text):
     return [x + newline for x in text.split(newline)], newline.decode()
 
 
-def escapeforpo(line):
+def escapehandler(match: re.Match) -> str:
+    return po_escape_map[match.group(0)]
+
+
+def escapeforpo(line: str) -> str:
     r"""
     Escapes a line for po format. assumes no \n occurs in the line.
 
     :param line: unescaped text
     """
-    special_locations = []
-    for special_key in po_escape_map:
-        special_locations.extend(quote.find_all(line, special_key))
-    special_locations = sorted(dict.fromkeys(special_locations).keys())
-    escaped_line = []
-    last_location = 0
-    for location in special_locations:
-        escaped_line.append(line[last_location:location])
-        escaped_line.append(po_escape_map[line[location : location + 1]])
-        last_location = location + 1
-    escaped_line.append(line[last_location:])
-    return "".join(escaped_line)
-
-
-def unescapehandler(escape):
-    return po_unescape_map.get(escape, escape)
-
-
-WIDE_CHARS = {"F", "W"}
-
-
-def cjklen(text: str) -> int:
-    """
-    Return the real width of an unicode text, the len of any other type.
-
-    Fullwidth and Wide CJK chars are double-width.
-    """
-    return sum(
-        2 if unicodedata.east_asian_width(char) in WIDE_CHARS else 1 for char in text
-    )
+    return po_escape_re.sub(escapehandler, line)
 
 
 def cjkslices(text: str, index: int) -> tuple[str, str]:
     """Return the two slices of a text cut to the index."""
-    if cjklen(text) <= index:
+    text_width = wcswidth(text)
+    if text_width == len(text) or text_width <= index:
         return text, ""
     length = 0
     i = 0
     for i in range(len(text)):
-        length += cjklen(text[i])
+        length += wcwidth(text[i])
         if length > index:
             break
     return text[:i], text[i:]
@@ -141,10 +126,11 @@ class PoWrapper(textwrap.TextWrapper):
     wordsep_re = re.compile(
         r"""
             (
+            \[[^\]]{1,73}\]|                      # [] braces
+            \\"[^"]{1,73}\\"|                     # quoted string
             \s+|                                  # any whitespace
-            [a-z0-9A-Z_-]+/|                      # nicely split long URLs
+            [a-z0-9A-Z_#\[\].-]+/|                # nicely split long URLs
             \w*\\.\w*|                            # any escape should not be split
-            \.(?=\w)|                             # full stop inside word
             [\w\!\'\&\.\,\?=<>%]+\s+|             # space should go with a word
             [^\s\w]*\w+[a-zA-Z]-(?=\w+[a-zA-Z])|  # hyphenated words
             (?<=[\w\!\"\'\&\.\,\?])-{2,}(?=\w)    # em-dash
@@ -153,7 +139,7 @@ class PoWrapper(textwrap.TextWrapper):
         re.VERBOSE,
     )
 
-    def __init__(self, width=77):
+    def __init__(self, width: int = 77) -> None:
         super().__init__(
             width=width,
             replace_whitespace=False,
@@ -162,27 +148,31 @@ class PoWrapper(textwrap.TextWrapper):
             break_long_words=True,
         )
 
+    def __eq__(self, other) -> bool:
+        if not isinstance(other, PoWrapper):
+            return False
+        return self.width == other.width
+
     def _handle_long_word(
         self, reversed_chunks: list[str], cur_line: list[str], cur_len: int, width: int
-    ):
+    ) -> None:
         """
         Handle a chunk of text (most likely a word, not whitespace) that
         is too long to fit in any line.
         """
-        # Figure out when indent is larger than the specified width, and make
-        # sure at least one character is stripped off on every pass
-        space_left = 1 if width < 1 else width - cur_len
-
         # We're allowed to break long words, then do so: put as much
         # of the next chunk onto the current line as will fit.
-        chunk_start, chunk_end = cjkslices(reversed_chunks[-1], space_left)
+        chunk_start, chunk_end = cjkslices(reversed_chunks[-1], width - cur_len)
         cur_line.append(chunk_start)
         reversed_chunks[-1] = chunk_end
 
     def _wrap_chunks(self, chunks: list[str]) -> list[str]:
         lines = []
-        if self.width <= 1:
-            raise ValueError("invalid width %r (must be > 1)" % self.width)
+        width = self.width
+        if width <= 1:
+            raise ValueError(f"invalid width {self.width!r} (must be > 1)")
+        if self.subsequent_indent or self.initial_indent:
+            raise ValueError("PoWrapper does not support indent")
 
         # Arrange in reverse order so items can be efficiently popped
         # from a stack of chucks.
@@ -194,14 +184,8 @@ class PoWrapper(textwrap.TextWrapper):
             cur_line = []
             cur_len = 0
 
-            # Figure out which static string will prefix this line.
-            indent = self.subsequent_indent if lines else self.initial_indent
-
-            # Maximum width for this line.
-            width = self.width - len(indent)
-
             while chunks:
-                l = cjklen(chunks[-1])
+                l = wcswidth(chunks[-1])
 
                 # Can at least squeeze this chunk onto the current line.
                 if cur_len + l <= width:
@@ -214,17 +198,17 @@ class PoWrapper(textwrap.TextWrapper):
 
             # The current line is full, and the next chunk is too big to
             # fit on *any* line (not just this one).
-            if chunks and cjklen(chunks[-1]) > width:
+            if not cur_line and chunks and wcswidth(chunks[-1]) > width:
                 self._handle_long_word(chunks, cur_line, cur_len, width)
 
             if cur_line:
                 # Convert current line back to a string and store it in
                 # list of all lines (return value).
-                lines.append(indent + "".join(cur_line))
+                lines.append("".join(cur_line))
         return lines
 
 
-def quoteforpo(text, wrapper_obj=None):
+def quoteforpo(text: str | None, wrapper_obj: PoWrapper | None = None) -> list[str]:
     """Quotes the given text for a PO file, returning quoted and escaped lines."""
     if text is None:
         return []
@@ -232,7 +216,7 @@ def quoteforpo(text, wrapper_obj=None):
         wrapper_obj = PoWrapper()
     text = escapeforpo(text)
     if wrapper_obj.width == -1:
-        return ['"%s"' % text]
+        return [f'"{text}"']
     lines = text.split("\\n")
     for i, l in enumerate(lines[:-1]):
         lines[i] = l + "\\n"
@@ -251,46 +235,33 @@ def quoteforpo(text, wrapper_obj=None):
     return polines
 
 
-def unescape(line):
+def unescapehandler(match: re.Match) -> str:
+    value = match.group(0)
+    try:
+        return po_unescape_map[value]
+    except KeyError:
+        return value[1]
+
+
+def unescape(line: str) -> str:
     """
     Unescape the given line.
 
     Quotes on either side should already have been removed.
     """
-    escape_places = quote.find_all(line, "\\")
-    if not escape_places:
-        return line
-
-    # filter escaped escapes
-    true_escape = False
-    true_escape_places = []
-    for escape_pos in escape_places:
-        true_escape = not true_escape if escape_pos - 1 in escape_places else True
-        if true_escape:
-            true_escape_places.append(escape_pos)
-
-    extracted = []
-    lastpos = 0
-    for pos in true_escape_places:
-        # everything leading up to the escape
-        extracted.append(line[lastpos:pos])
-        # the escaped sequence (consuming 2 characters)
-        extracted.append(unescapehandler(line[pos : pos + 2]))
-        lastpos = pos + 2
-
-    extracted.append(line[lastpos:])
-    return "".join(extracted)
+    return po_unescape_re.sub(unescapehandler, line)
 
 
-def unquotefrompo(postr):
+def unquotefrompo(postr: str) -> str:
     return "".join(unescape(line[1:-1]) for line in postr)
 
 
-def is_null(lst):
-    return lst == [] or len(lst) == 1 and lst[0] == '""'
+def is_null(lst: list[str]) -> bool:
+    lst_len = len(lst)
+    return lst_len == 0 or (lst_len == 1 and lst[0] == '""')
 
 
-def extractstr(string):
+def extractstr(string: str) -> str:
     left = string.find('"')
     right = string.rfind('"')
     if right > -1:
@@ -471,7 +442,7 @@ class pounit(pocommon.pounit):
         newline = self.newline
         if origin == "translator" or origin is None:
             parts.append(comment[2:] or newline for comment in self.othercomments)
-        if origin in ["programmer", "developer", "source code", None]:
+        if origin in {"programmer", "developer", "source code", None}:
             parts.append(comment[3:] or newline for comment in self.automaticcomments)
         if not parts:
             raise ValueError("Comment type not valid")
@@ -491,7 +462,7 @@ class pounit(pocommon.pounit):
         commentlist = self.othercomments
         linestart = "#"
         autocomments = False
-        if origin in ["programmer", "developer", "source code"]:
+        if origin in {"programmer", "developer", "source code"}:
             autocomments = True
             commentlist = self.automaticcomments
             linestart = "#."
@@ -502,7 +473,7 @@ class pounit(pocommon.pounit):
         if position == "append":
             newcomments = commentlist + newcomments
         elif position == "prepend":
-            newcomments = newcomments + commentlist
+            newcomments += commentlist
 
         if autocomments:
             self.automaticcomments = newcomments
@@ -633,20 +604,18 @@ class pounit(pocommon.pounit):
         return (
             is_null(self.msgid)
             and not is_null(self.msgstr)
-            and self.msgidcomments == []
+            and len(self.msgidcomments) == 0
             and is_null(self.msgctxt)
         )
 
     def isblank(self):
-        if self.isheader() or self.msgidcomments:
-            return False
-        if (
-            (self._msgidlen() == 0)
+        return (
+            not self.isheader()
+            and not self.msgidcomments
+            and (self._msgidlen() == 0)
             and (self._msgstrlen() == 0)
             and (is_null(self.msgctxt))
-        ):
-            return True
-        return False
+        )
         # TODO: remove:
         # Before, the equivalent of the following was the final return statement:
         # return len(self.source.strip()) == 0
@@ -675,7 +644,7 @@ class pounit(pocommon.pounit):
 
                 # (commentmarker) ...
         """
-        commentmarker = "(%s)" % commentmarker
+        commentmarker = f"({commentmarker})"
         for comment in self.othercomments:
             if comment.replace("#", "", 1).strip().startswith(commentmarker):
                 return True
@@ -702,7 +671,7 @@ class pounit(pocommon.pounit):
     def markfuzzy(self, present=True):
         if present:
             self.set_state_n(self.STATE[self.S_FUZZY][0])
-        elif self.hasplural() and not self._msgstrlen() or is_null(self.msgstr):
+        elif (self.hasplural() and not self._msgstrlen()) or is_null(self.msgstr):
             self.set_state_n(self.STATE[self.S_UNTRANSLATED][0])
         else:
             self.set_state_n(self.STATE[self.S_TRANSLATED][0])
@@ -765,13 +734,12 @@ class pounit(pocommon.pounit):
                 combinedcomment = []
                 for comment in partcomments:
                     comment = unquotefrompo([comment])
-                    if comment.startswith("_:"):
-                        comment = comment[len("_:") :]
+                    comment = comment.removeprefix("_:")
                     if comment.endswith("\\n"):
                         comment = comment[: -len("\\n")]
                     # Before we used to strip. Necessary in some cases?
                     combinedcomment.append(comment)
-                partcomments = self.quote("_:%s" % "".join(combinedcomment))
+                partcomments = self.quote("_:{}".format("".join(combinedcomment)))
                 # Strip heading empty line for multiline string, it was already added above
                 if partcomments[0] == '""':
                     partcomments = partcomments[1:]
@@ -893,7 +861,7 @@ class pounit(pocommon.pounit):
 
     def setmsgidcomment(self, msgidcomment):
         if msgidcomment:
-            self.msgidcomments = ['"_: %s\\n"' % msgidcomment]
+            self.msgidcomments = [f'"_: {msgidcomment}\\n"']
         else:
             self.msgidcomments = []
 
@@ -965,7 +933,9 @@ class pofile(pocommon.pofile):
         markedpos = []
 
         def addcomment(thepo):
-            thepo.msgidcomments.append('"_: %s\\n"' % " ".join(thepo.getlocations()))
+            thepo.msgidcomments.append(
+                '"_: {}\\n"'.format(" ".join(thepo.getlocations()))
+            )
             markedpos.append(thepo)
 
         for thepo in self.units:
@@ -984,11 +954,11 @@ class pofile(pocommon.pofile):
                     origpo = id_dict[id]
                     if origpo not in markedpos and not origpo.msgctxt:
                         origpo.msgctxt.append(
-                            '"%s"' % escapeforpo(" ".join(origpo.getlocations()))
+                            '"{}"'.format(escapeforpo(" ".join(origpo.getlocations())))
                         )
                         markedpos.append(thepo)
                     thepo.msgctxt.append(
-                        '"%s"' % escapeforpo(" ".join(thepo.getlocations()))
+                        '"{}"'.format(escapeforpo(" ".join(thepo.getlocations())))
                     )
                     if thepo.msgctxt != id_dict[id].msgctxt:
                         uniqueunits.append(thepo)
@@ -1004,7 +974,7 @@ class pofile(pocommon.pofile):
                         addcomment(thepo)
                     else:
                         thepo.msgctxt.append(
-                            '"%s"' % escapeforpo(" ".join(thepo.getlocations()))
+                            '"{}"'.format(escapeforpo(" ".join(thepo.getlocations())))
                         )
                 id_dict[id] = thepo
                 uniqueunits.append(thepo)
@@ -1034,9 +1004,7 @@ class pofile(pocommon.pofile):
                 yield unit
 
     def addunit(self, unit):
-        needs_update = (
-            unit.wrapper and self.wrapper and (unit.wrapper.width != self.wrapper.width)
-        )
+        needs_update = unit.wrapper != self.wrapper
         unit.wrapper = self.wrapper
         super().addunit(unit)
         if needs_update:
