@@ -73,7 +73,7 @@ def qmunpack(file_="messages.qm"):
     """Helper to unpack Qt .qm files into a Python string."""
     with open(file_, "rb") as fh:
         s = fh.read()
-        print("\\x%02x" * len(s) % tuple(map(ord, s)))
+        return "\\x%02x" * len(s) % tuple(map(ord, s))
 
 
 class qmunit(base.TranslationUnit):
@@ -118,7 +118,7 @@ class qmfile(base.TranslationStore):
         sectionheader = 5
 
         def section_debug(name, section_type, startsection, length):
-            print(
+            print(  # noqa: T201
                 "Section: %s (type: %#x, offset: %#x, length: %d)"
                 % (name, section_type, startsection, length)
             )
@@ -161,7 +161,7 @@ class qmfile(base.TranslationStore):
         while pos < messages_start + len(messages_data):
             (subsection,) = struct.unpack(">B", input[pos : pos + 1])
             if subsection == 0x01:  # End
-                pos = pos + 1
+                pos += 1
                 if source is not None and target is not None:
                     newunit = self.addsourceunit(source)
                     newunit.target = target
@@ -169,7 +169,7 @@ class qmfile(base.TranslationStore):
                 else:
                     raise ValueError("Old .qm format with no source defined")
                 continue
-            pos = pos + 1
+            pos += 1
             (length,) = struct.unpack(">l", input[pos : pos + 4])
             if subsection == 0x03:  # Translation
                 if length != -1:
@@ -178,13 +178,13 @@ class qmfile(base.TranslationStore):
                     )
                     string, templen = codecs.utf_16_be_decode(raw)
                     if target:
-                        target.strings.append(string)
+                        target.extra_strings.append(string)
                     else:
                         target = multistring(string)
                     pos = pos + 4 + length
                 else:
                     target = ""
-                    pos = pos + 4
+                    pos += 4
             elif subsection == 0x06:  # SourceText
                 source = input[pos + 4 : pos + 4 + length].decode("iso-8859-1")
                 pos = pos + 4 + length
@@ -196,7 +196,7 @@ class qmfile(base.TranslationStore):
                 pos = pos + 4 + length
             elif subsection == 0x05:  # hash
                 # hash = input[pos:pos + 4]
-                pos = pos + 4
+                pos += 4
             else:
                 if subsection == 0x02:  # SourceText16
                     subsection_name = "SourceText16"
