@@ -30,6 +30,7 @@ from translate.misc.multistring import multistring
 from translate.misc.xml_helpers import (
     clear_content,
     getXMLspace,
+    safely_set_text,
     setXMLlang,
     setXMLspace,
 )
@@ -48,46 +49,6 @@ ID_SEPARATOR = "\04"
 ID_SEPARATOR_SAFE = "__%04__"
 
 
-# List of ASCII control codes which are unaccepted in XML.
-ASCII_CONTROL_CODES = [
-    "0000",  # Unicode Character 'NULL' (U+0000)
-    "0001",  # Unicode Character 'START OF HEADING' (U+0001)
-    "0002",  # Unicode Character 'START OF TEXT' (U+0002)
-    "0003",  # Unicode Character 'END OF TEXT' (U+0003)
-    "0004",  # Unicode Character 'END OF TRANSMISSION' (U+0004)
-    "0005",  # Unicode Character 'ENQUIRY' (U+0005)
-    "0006",  # Unicode Character 'ACKNOWLEDGE' (U+0006)
-    "0007",  # Unicode Character 'BELL' (U+0007), "\a" in Python
-    "0008",  # Unicode Character 'BACKSPACE' (U+0008), "\b" in Python
-    "000b",  # Unicode Character 'LINE TABULATION' (U+000B), "\v" in Python
-    "000c",  # Unicode Character 'FORM FEED (FF)' (U+000C), "\f" in Python
-    "000e",  # Unicode Character 'SHIFT OUT' (U+000E)
-    "000f",  # Unicode Character 'SHIFT IN' (U+000F)
-    "0010",  # Unicode Character 'DATA LINK ESCAPE' (U+0010)
-    "0011",  # Unicode Character 'DEVICE CONTROL ONE' (U+0011)
-    "0012",  # Unicode Character 'DEVICE CONTROL TWO' (U+0012)
-    "0013",  # Unicode Character 'DEVICE CONTROL THREE' (U+0013)
-    "0014",  # Unicode Character 'DEVICE CONTROL FOUR' (U+0014)
-    "0015",  # Unicode Character 'NEGATIVE ACKNOWLEDGE' (U+0015)
-    "0016",  # Unicode Character 'SYNCHRONOUS IDLE' (U+0016)
-    "0017",  # Unicode Character 'END OF TRANSMISSION BLOCK' (U+0017)
-    "0018",  # Unicode Character 'CANCEL' (U+0018)
-    "0019",  # Unicode Character 'END OF MEDIUM' (U+0019)
-    "001a",  # Unicode Character 'SUBSTITUTE' (U+001A)
-    "001b",  # Unicode Character 'ESCAPE' (U+001B)
-    "001c",  # Unicode Character 'INFORMATION SEPARATOR FOUR' (U+001C)
-    "001d",  # Unicode Character 'INFORMATION SEPARATOR THREE' (U+001D)
-    "001e",  # Unicode Character 'INFORMATION SEPARATOR TWO' (U+001E)
-    "001f",  # Unicode Character 'INFORMATION SEPARATOR ONE' (U+001F)
-]
-
-ASCII_CONTROL_CHARACTERS = {code: chr(int(code, 16)) for code in ASCII_CONTROL_CODES}
-
-ASCII_CONTROL_CHARACTERS_ESCAPES = {
-    code: "&#x%s;" % code.lstrip("0") or "0" for code in ASCII_CONTROL_CODES
-}
-
-
 class xliffunit(lisa.LISAunit):
     """A single term in the xliff file."""
 
@@ -136,15 +97,6 @@ class xliffunit(lisa.LISAunit):
             return
         setXMLspace(self.xmlelement, "preserve")
 
-    def getNodeText(self, languageNode, xml_space="preserve"):
-        """Retrieves the term from the given :attr:`languageNode`."""
-        text = super().getNodeText(languageNode, xml_space=xml_space)
-        if text is not None:
-            # Unescape the unaccepted ASCII control characters.
-            for code, character in ASCII_CONTROL_CHARACTERS.items():
-                text = text.replace(ASCII_CONTROL_CHARACTERS_ESCAPES[code], character)
-        return text
-
     def createlanguageNode(self, lang, text, purpose):
         """Returns an xml Element setup with given parameters."""
         # TODO: for now we do source, but we have to test if it is target,
@@ -156,11 +108,8 @@ class xliffunit(lisa.LISAunit):
         # TODO: check language
         # setXMLlang(langset, lang)
 
-        # Escape the unaccepted ASCII control characters.
-        for code in ASCII_CONTROL_CODES:
-            text = text.replace(chr(int(code, 16)), "&#x%s;" % code.lstrip("0") or "0")
+        safely_set_text(langset, text)
 
-        langset.text = text
         return langset
 
     def getlanguageNodes(self):
@@ -195,7 +144,7 @@ class xliffunit(lisa.LISAunit):
     @property
     def rich_source(self):
         # rsrc = xml_to_strelem(self.source_dom)
-        # logging.debug('rich source: %s' % (repr(rsrc)))
+        # logger.debug('rich source: %s' % (repr(rsrc)))
         # from dubulib.debug.misc import print_stack_funcs
         # print_stack_funcs()
         return [
@@ -214,6 +163,7 @@ class xliffunit(lisa.LISAunit):
             self.set_target_dom(self.createlanguageNode(lang, "", "target"))
             return
 
+        self._ensure_xml_space_preserve()
         languageNode = self.get_target_dom()
         if languageNode is None:
             languageNode = self.createlanguageNode(lang, "", "target")
@@ -460,12 +410,15 @@ class xliffunit(lisa.LISAunit):
             if state_id < self.S_UNREVIEWED:
                 self.set_state_n(self.S_UNREVIEWED)
 
+    def _ensure_xml_space_preserve(self):
+        if getXMLspace(self.xmlelement) != "preserve":
+            setXMLspace(self.xmlelement, "preserve")
+
     def settarget(self, target, lang="xx", append=False):
         """Sets the target string to the given value."""
         super().settarget(target, lang, append)
         if target:
-            if getXMLspace(self.xmlelement) != "preserve":
-                setXMLspace(self.xmlelement, "preserve")
+            self._ensure_xml_space_preserve()
             self.marktranslated()
 
     # This code is commented while this will almost always return false.
@@ -477,9 +430,7 @@ class xliffunit(lisa.LISAunit):
 
     def istranslatable(self):
         value = self.xmlelement.get("translate")
-        if value and value.lower() == "no":
-            return False
-        return True
+        return not value or value.lower() != "no"
 
     def marktranslated(self):
         state_id = self.get_state_id()
@@ -657,7 +608,13 @@ class xlifffile(lisa.LISAfile):
 </body>
 </file>
 </xliff>"""
-    XMLindent = {"indent": "  ", "max_level": 4, "toplevel": False}
+    XMLindent = {
+        "indent": "  ",
+        "max_level": 8,
+        "leaves": {"note", "source", "target"},
+        "toplevel": False,
+        "ignore_preserve": {"trans-unit"},
+    }
     namespace = "urn:oasis:names:tc:xliff:document:1.1"
     unversioned_namespace = "urn:oasis:names:tc:xliff:document:"
 
@@ -735,7 +692,7 @@ class xlifffile(lisa.LISAfile):
         filenodes = self.document.getroot().iterchildren(self.namespaced("file"))
         filenames = [self.getfilename(filenode) for filenode in filenodes]
         filenames = list(filter(None, filenames))
-        if len(filenames) == 1 and filenames[0] == "":
+        if len(filenames) == 1 and not filenames[0]:
             filenames = []
         return filenames
 
